<?php
/*
   TM Offline Stats v1.1
   Copyright (c) 2006-2009 Alexander Domnin

   This program is licenced under the
   Creative Commons Attribution-Noncommercial-No Derivative Works 3.0 Unported.
   To view a copy of this licence, visit http://creativecommons.org/licenses/by-nc-nd/3.0/

   This program is distributed in the hope that it will be useful, but WITHOUT ANY WARRANTY;
   without even the implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
*/

require_once 'tmos_funcs.inc.php';

// *****************************************************************************
// Ttmos_TI
// *****************************************************************************

class Ttmos_TI {

   function parse_xmlheader(&$xmlheader) {
      $vals = array();
      $tags = array();

      // ????
      $xmlheader = str_replace('', '?', $xmlheader);

      $xml_parser = xml_parser_create('UTF-8');
      xml_parser_set_option($xml_parser, XML_OPTION_SKIP_WHITE, 0);
      xml_parse_into_struct($xml_parser, $xmlheader, $vals, $tags);
      xml_parser_free($xml_parser);

      if (isset($tags['HEADER'][0]) &&
          isset($tags['IDENT'][0]) &&
          isset($tags['DESC'][0]) &&
          isset($tags['TIMES'][0]) &&
          isset($vals[$tags['HEADER'][0]]['attributes']['TYPE']) &&
          ($vals[$tags['HEADER'][0]]['attributes']['TYPE'] == "challenge") &&
          isset($vals[$tags['HEADER'][0]]['attributes']['VERSION']) &&
          isset($vals[$tags['IDENT'][0]]['attributes']['UID']) &&
          isset($vals[$tags['IDENT'][0]]['attributes']['NAME']) &&
          isset($vals[$tags['IDENT'][0]]['attributes']['AUTHOR']) &&
          isset($vals[$tags['DESC'][0]]['attributes']['ENVIR']) &&
          isset($vals[$tags['DESC'][0]]['attributes']['MOOD']) &&
          isset($vals[$tags['DESC'][0]]['attributes']['TYPE']) &&
          isset($vals[$tags['DESC'][0]]['attributes']['NBLAPS']) &&
          isset($vals[$tags['DESC'][0]]['attributes']['PRICE']) &&
          isset($vals[$tags['TIMES'][0]]['attributes']['BRONZE']) &&
          isset($vals[$tags['TIMES'][0]]['attributes']['SILVER']) &&
          isset($vals[$tags['TIMES'][0]]['attributes']['GOLD']) &&
          isset($vals[$tags['TIMES'][0]]['attributes']['AUTHORSCORE']) &&
          isset($vals[$tags['TIMES'][0]]['attributes']['AUTHORTIME']) &&
          ($vals[$tags['DESC'][0]]['attributes']['TYPE'] == "Race")) {

         $result = array();
         $result['b'] = $vals[$tags['IDENT'][0]]['attributes']['UID'];
         $result['c'] = urldecode($vals[$tags['IDENT'][0]]['attributes']['NAME']);
         $result['d'] = name_clear($result['c']);
         $result['e'] = urldecode($vals[$tags['IDENT'][0]]['attributes']['AUTHOR']);
         $result['f'] = envir_toid($vals[$tags['DESC'][0]]['attributes']['ENVIR']);
         $result['g'] = trim($vals[$tags['DESC'][0]]['attributes']['MOOD']);
         if (preg_match("/^\d{2}x\d{2}(\w+)/i", $result['g'], $matches))
            $result['g'] = ucfirst($matches[1]);
         $result['h'] = $vals[$tags['DESC'][0]]['attributes']['TYPE'];
         $result['i'] = $vals[$tags['DESC'][0]]['attributes']['NBLAPS']; // > 0 = ml
         $result['j'] = $vals[$tags['DESC'][0]]['attributes']['PRICE'];
         $result['k'] = time_msectofull($vals[$tags['TIMES'][0]]['attributes']['BRONZE']);
         $result['l'] = time_msectofull($vals[$tags['TIMES'][0]]['attributes']['SILVER']);
         $result['m'] = time_msectofull($vals[$tags['TIMES'][0]]['attributes']['GOLD']);
         $result['n'] = time_msectofull($vals[$tags['TIMES'][0]]['attributes']['AUTHORTIME']);
         $result['o'] = $vals[$tags['TIMES'][0]]['attributes']['AUTHORSCORE'];
         if ($result['o'] < 0) $result['o'] = 0;
         $result['r'] = ''; // filename
         $result['s'] = game_fromenvir($vals[$tags['HEADER'][0]]['attributes']['VERSION'],
                                       $vals[$tags['DESC'][0]]['attributes']['ENVIR']);
      }
      else 
         $result = -1;

      return $result;
   }

   // based on http://en.tm-wiki.org/wiki/GBX
   function parse_hexheader(&$hexheader) {
      // check signature and gbx type
      if (((substr($hexheader, 0, 9) != 'GBX'."\x06\x00".'BUCR') &&
           (substr($hexheader, 0, 9) != 'GBX'."\x06\x00".'BUCE')) ||
          ((substr($hexheader, 9, 4) != "\x00\x30\x00\x24") &&
           (substr($hexheader, 9, 4) != "\x00\x30\x04\x03"))) {
         return -1;
      }
      $ver = str_toint(substr($hexheader, 17, 4)); // challenge version
      // times/info block ...
      if ($ver >= 5)
         $offset = 61;
      else if ($ver >= 4)
         $offset = 53;
      else if ($ver >= 3)
         $offset = 45;
      else
         $offset = 37;
      $ver2 = str_toint($hexheader[$offset], 1); // "exe version"
      // check for Race
      if ($ver2 >= 6 && str_toint(substr($hexheader, $offset + 29, 4)) != 0) {
         return -1;
      }
      $result = array();
      $result['k'] = time_msectofull(str_toint(substr($hexheader, $offset  + 5, 4))); // bronze
      $result['l'] = time_msectofull(str_toint(substr($hexheader, $offset  + 9, 4))); // silver
      $result['m'] = time_msectofull(str_toint(substr($hexheader, $offset + 13, 4))); // gold
      $result['n'] = time_msectofull(str_toint(substr($hexheader, $offset + 17, 4))); // author
      if ($ver >= 3)
         $result['j'] = str_toint(substr($hexheader, $offset + 21, 4)); // price
      else
         $result['j'] = 0;
      if ($ver >= 3 && $ver2 >= 6) {
         $result['i'] = str_toint(substr($hexheader, $offset + 25, 4)); // nblaps > 0 = ml
         $result['h'] = str_toint(substr($hexheader, $offset + 29, 4)); // type
         if ($result['h'] == 0)
            $result['h'] = 'Race';
         else if ($result['h'] == 1)
            $result['h'] = 'Platform';
         else if ($result['h'] == 2)
            $result['h'] = 'Puzzle';
         else if ($result['h'] == 3)
            $result['h'] = 'Crazy';
         else if ($result['h'] == 4)
            $result['h'] = 'Stunts';
      }
      else {
         $result['i'] = 0;
         $result['h'] = '';
      }
      if ($ver >= 4)
         $result['o'] = str_toint(substr($hexheader, $offset + 37, 4)); // authorscore
      else
         $result['o'] = 0;
      // strings block ...
      $offset += str_toint(substr($hexheader, 25, 4));
      if ($ver >= 3)
         $offset += 9;
      else
         $offset += 5;
      $len = str_toint(substr($hexheader, $offset, 4)); // uid len
      $result['b'] = substr($hexheader, $offset + 4, $len); // uid
      $offset += 8 + $len;
      $len = str_toint(substr($hexheader, $offset, 4)); // envir len
      $envir = substr($hexheader, $offset + 4, $len);
      $result['f'] = envir_toid($envir); // envir
      $offset += 8 + $len;
      $len = str_toint(substr($hexheader, $offset, 4)); // author len
      $result['e'] = substr($hexheader, $offset + 4, $len); // author
      if (substr($result['e'], 0, 3) == "\xEF\xBB\xBF")  // del BOM
         $result['e'] = substr($result['e'], 3);
      $offset += 4 + $len;
      $len = str_toint(substr($hexheader, $offset, 4)); // track len
      $result['c'] = substr($hexheader, $offset + 4, $len); // track
      if (substr($result['c'], 0, 3) == "\xEF\xBB\xBF") // del BOM
         $result['c'] = substr($result['c'], 3);
      $result['d'] = name_clear($result['c']);
      if ($ver >= 4) {
         $offset += 9 + $len;
         $len = str_toint(substr($hexheader, $offset, 4)); // passw len
         $offset += 8 + $len;
         $len = str_toint(substr($hexheader, $offset, 4)); // mood len
         $result['g'] = substr($hexheader, $offset + 4, $len); // mood
         if (preg_match("/^\d{2}x\d{2}(\w+)/i", $result['g'], $matches))
            $result['g'] = ucfirst($matches[1]);
      }
      else
         $result['g'] = '';
      // other
      $result['r'] = ''; // filename
      $result['s'] = game_fromenvir($ver, $envir); // game  ver = 5 => united

      return $result;
   }

   function cmp($a, $b) {
      return strcasecmp($a['d'], $b['d']);
   }

   function get_info($dirs, $sort = false) {
      $result = array();

      for ($i = 0; $i < $dirs['count']; $i++) {
         $list = @filelist_ex($dirs[$i]['directory'], '.*\.gbx');
         foreach ($list as $filename) {
            // xml header
            $buf = @filedscr_ex($filename, '<header', '/header>', false);
            if ($buf != -1) {
               $buf = $this->parse_xmlheader($buf);
            }
            // hex header
            if ($buf == -1) {
               $buf = @filedscr_ex($filename, 0, 4095, false);
               if ($buf != -1) {
                  $buf = $this->parse_hexheader($buf);
               }
            }
            if ($buf !== -1) {
               $buf['r'] = ltrim(str_replace($dirs[$i]['directory'], '', $filename), '/\\ ');
               $buf['t'] = ($dirs[$i]['directory'] == DIR_FILES);
               $result[id_get($buf['b'])] = $buf;
            }
         }
      }

      if ($sort) uasort($result, array("Ttmos_TI", "cmp"));

      return $result;
   }

   function get_thumbs($dir) {
      $result = array();

      $list = @filelist_ex($dir, '.*\.gbx');
      foreach ($list as $filename) {
         $buf = @filedscr_ex($filename, '<header', '/header>', false);
         if ($buf != -1) {
            $uid = rtrim(substr($buf, strpos($buf, 'uid="') + 5, 27), '" ');
            $buf = @filedscr_ex($filename, '<Thumbnail.jpg>', '</Thumbnail.jpg>', true, 32768);
            if ($buf != -1) {
               $result[$uid] = $buf;
            }
         }
      }

      return $result;
   }

} // class

?>